#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fstream>
#include <SDL.h>
#include <SDL_ttf.h>
#include <SDL_image.h>
#include <SDL_mixer.h>
#include <SDL_framerate.h>
#include <math.h>
#include "shooter.h"
#include "CInput.h"
#include "CUsbJoystickSetup.h"
#include "Functions.h"
#include "CGameObject.h"
#include "CSinCosTable.h"
#include "CWorldParts.h"
#include "CBullet.h"
#include "CEnemy.h"

using namespace std;


enum GameStates {GSQuit,GSIntro,GSGame,GSLevelEditor,GSTitleScreen};



SDL_Surface *Buffer,*Screen;
TTF_Font* font;
SDL_Joystick *Joystick;
bool GlobalSoundEnabled = true,TvOutMode=false;
GameStates GameState = GSLevelEditor;
int Volume = 128;
FPSmanager Fpsman;
CSinCosTable *SinCosTable;
SDL_Surface *IMGStar1,*IMGStar2,*IMGStar3, *IMGSquare, *IMGCircle, *IMGPolygon, *IMGBulletPlayer, *IMGBulletEnemy, *IMGTriangle, *IMGExplosion, *IMGBackgroundStar1 , *IMGBackgroundStar2,
            *IMGBackgroundGrass,*IMGBackgroundWater,*IMGBackgroundEarth,*IMGBackgroundWaterGrass,*IMGBackgroundGrassWater, *IMGBackgroundGrassEarth,*IMGBackgroundEarthGrass,
            *IMGBackgroundEarthPond;
CSurfaceRotationCache *Star1,*Star2,*Star3, *Square, *Circle, *Polygon, *Triangle;
CWorldParts *WorldParts;

// set the volume of the music and both channels (left/right)
void SetVolume(const int VolumeIn)
{
	Mix_Volume(0,VolumeIn);
	Mix_Volume(1,VolumeIn);
	Mix_VolumeMusic(VolumeIn);
}

// increase the volume with 8
void IncVolume()
{
	if (Volume < 128)
	{
		Volume += 8;
		SetVolume(Volume);
	}
}

// decrease the volume with 8
void DecVolume()
{
	if (Volume > 0)
	{
		Volume-= 8;
		SetVolume(Volume);
	}
}

void LoadSettings()
{
 	FILE *SettingsFile;
 	SettingsFile = fopen("./settings.dat","r");
 	if(SettingsFile)
 	{
		fscanf(SettingsFile,"Volume=%d\n",&Volume);
		SetVolume(Volume);
		fclose(SettingsFile);
 	}
 	else
 	{
		SetVolume(128);
	}
}

void SaveSettings()
{
 	FILE *SettingsFile;
 	SettingsFile = fopen("./settings.dat","w");
 	if(SettingsFile)
 	{
		fprintf(SettingsFile,"Volume=%d\n",Volume);
		fclose(SettingsFile);
		sync();
 	}
}



void UnLoadGraphics()
{
    if (IMGStar1)
        SDL_FreeSurface(IMGStar1);
    if (IMGStar2)
        SDL_FreeSurface(IMGStar2);
    if (IMGStar3)
        SDL_FreeSurface(IMGStar3);
    if (IMGSquare)
        SDL_FreeSurface(IMGSquare);
    if (IMGPolygon)
        SDL_FreeSurface(IMGPolygon);
    if (IMGCircle)
        SDL_FreeSurface(IMGCircle);
    if (IMGTriangle)
        SDL_FreeSurface(IMGTriangle);
    if (IMGBulletPlayer)
        SDL_FreeSurface(IMGBulletPlayer);
    if (IMGBulletEnemy)
        SDL_FreeSurface(IMGBulletEnemy);
    if(IMGExplosion)
        SDL_FreeSurface(IMGExplosion);
    if(IMGBackgroundStar1)
        SDL_FreeSurface(IMGBackgroundStar1);
    if(IMGBackgroundStar2)
        SDL_FreeSurface(IMGBackgroundStar2);

    if (IMGBackgroundGrass)
        SDL_FreeSurface(IMGBackgroundGrass);
    if (IMGBackgroundWater)
        SDL_FreeSurface(IMGBackgroundWater);
    if (IMGBackgroundEarth)
        SDL_FreeSurface(IMGBackgroundEarth);
    if (IMGBackgroundWaterGrass)
        SDL_FreeSurface(IMGBackgroundWaterGrass);
    if (IMGBackgroundGrassWater)
        SDL_FreeSurface(IMGBackgroundGrassWater);
    if (IMGBackgroundGrassEarth)
        SDL_FreeSurface(IMGBackgroundGrassEarth);
    if (IMGBackgroundEarthGrass)
        SDL_FreeSurface(IMGBackgroundEarthGrass);
    if(IMGBackgroundEarthPond)
        SDL_FreeSurface(IMGBackgroundEarthPond);


    delete Star1;
    delete Star2;
    delete Star3;
    delete Square;
    delete Circle;
    delete Triangle;
    delete Polygon;
}

void LoadGraphics()
{
    SDL_Surface *Tmp;

    Tmp = IMG_Load("./graphics/backgroundearthpond.png");
    if(Tmp)
    {
        //SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGBackgroundEarthPond = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/backgroundgrass.png");
    if(Tmp)
    {
      //  SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGBackgroundGrass = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/backgroundwater.png");
    if(Tmp)
    {
     //   SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGBackgroundWater = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/backgroundearth.png");
    if(Tmp)
    {
    //    SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGBackgroundEarth = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }


    Tmp = IMG_Load("./graphics/backgroundwatergrass.png");
    if(Tmp)
    {
   //     SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGBackgroundWaterGrass = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp  = IMG_Load("./graphics/backgroundgrasswater.png");
    if(Tmp)
    {
       // SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGBackgroundGrassWater = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/backgroundgrassearth.png");
    if(Tmp)
    {
       // SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGBackgroundGrassEarth = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/backgroundearthgrass.png");
    if(Tmp)
    {
      //  SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGBackgroundEarthGrass = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }


    Tmp = IMG_Load("./graphics/explosion.png");
    if(Tmp)
    {
        SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGExplosion = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/animstar1.png");
    if(Tmp)
    {
        SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGBackgroundStar1= SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        ColorImage(IMGBackgroundStar1,IMGBackgroundStar1,1.2,1.2,1.5);
    }

    Tmp = IMG_Load("./graphics/animstar2.png");
    if(Tmp)
    {
        SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGBackgroundStar2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        ColorImage(IMGBackgroundStar2,IMGBackgroundStar2,1.8,1.4,0.4);
    }


    Tmp = IMG_Load("./graphics/star1.png");
    if(Tmp)
    {
        SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGStar1 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/star2.png");
    if(Tmp)
    {
        SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGStar2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/star3.png");
    if(Tmp)
    {
        SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGStar3 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }


    Tmp = IMG_Load("./graphics/triangle.png");
    if(Tmp)
    {
        SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGTriangle = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/square.png");
    if(Tmp)
    {
        SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGSquare = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }



    Tmp = IMG_Load("./graphics/circle.png");
    if(Tmp)
    {
        SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGCircle = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/polygon.png");
    if(Tmp)
    {
        SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGPolygon = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/bullet2.png");
    if(Tmp)
    {
        SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGBulletPlayer = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        ColorImage(IMGBulletPlayer,IMGBulletPlayer,1.8,1.4,0);
    }

    Tmp = IMG_Load("./graphics/bullet.png");
    if(Tmp)
    {
        SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
        IMGBulletEnemy = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        ColorImage(IMGBulletEnemy,IMGBulletEnemy,1.8,0,0.4);
    }

    Star1 = new CSurfaceRotationCache(IMGStar1,0.4,1,1);
    Star2 = new CSurfaceRotationCache(IMGStar2,1.8,1,0.4);
    Star3 = new CSurfaceRotationCache(IMGStar3,1,0.8,1.2);
    Square = new CSurfaceRotationCache(IMGSquare,1.6,0,0.4);
    Circle = new CSurfaceRotationCache(IMGCircle,1.8,1.4,0.4);
    Triangle = new CSurfaceRotationCache(IMGTriangle,0.5,1.7,0.9);
    Polygon = new CSurfaceRotationCache(IMGPolygon,0.25,0.75,1);
}

void TitleScreen()
{
    SDL_Rect Rect;
    CInput *Input = new CInput(10);
    while (GameState == GSTitleScreen)
    {
        Input->Update();
        if (Input->Ready() && (Input->KeyboardHeld[DINGOO_BUTTON_SELECT] || Input->KeyboardHeld[SDLK_ESCAPE] || Input->SpecialsHeld[SPECIAL_QUIT_EV]))
            GameState = GSQuit;
        //tekenen naar buffer
        Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = 0;
        Rect.y = 0;
        SDL_FillRect(Buffer,NULL,SDL_MapRGB(Buffer->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,NULL);
        SDL_Flip(Screen);

    }
    delete Input;
}

void LevelEditor()
{
    SDL_Color Color = {255,255,255,255};
    CInput *Input = new CInput(5);
    int i,numtest= 100;
    char Text[100];
    int framecounter = 0, currentfps=0;
    int Time1 = 0;
    int Y= 120,X=160;
    WorldParts->SetLeveleditorMode(true);
    WorldParts->MapInc = 5;
    for (i = 0; i <numtest; i ++)
    {
          //  if(rand()%2 == 1)
          //      WorldParts->Add(new CGameObject(Star1,rand()%320,rand()%20,rand()%180,rand()%10+1,rand()%3+1,200,200,SinCosTable));
          //  else
          //      WorldParts->Add(new CGameObject(Star2,rand()%320,rand()%20,rand()%180,rand()%10+1,rand()%3+1,200,200,SinCosTable));
        if(i<10)
        {
            if(i%2 == 0)
                WorldParts->AddMapItem(160,i*10 +1,1,3,2,2.5,3);
            else
                WorldParts->AddMapItem(160,i*10 +1,1,3,3,2.5,3);
        }
        else
            if (i < 60)
            {
                if(i%2 == 0)
                    WorldParts->AddMapItem(80+(rand()%20),i*10 + 101 ,rand()%7+1,3,2,2.5,5);
                else
                    WorldParts->AddMapItem(80-(rand()%20),i*10 + 101,rand()%7+1,3,3,2.5,5);
            }
            else
                {
                    if(i%2 == 0)
                        WorldParts->AddMapItem(240+(rand()%20),i*10 + 201,rand()%7+1,3,2,3,5);
                    else
                        WorldParts->AddMapItem(240-(rand()%20),i*10 + 201,rand()%7+1,3,3,3,5);
                }

    }
    WorldParts->AddBackGroundPart(1);
    WorldParts->AddBackGroundPart(4);
    WorldParts->AddBackGroundPart(2);
    WorldParts->AddBackGroundPart(6);
    WorldParts->AddBackGroundPart(3);
    WorldParts->AddBackGroundPart(3);
     //find better way

    while (GameState == GSLevelEditor)
    {
        Input->Update();
        if (Input->Ready() && (Input->KeyboardHeld[DINGOO_BUTTON_SELECT] || Input->KeyboardHeld[SDLK_ESCAPE] ||Input->SpecialsHeld[SPECIAL_QUIT_EV]) )
            GameState = GSGame;

        if ((Input->KeyboardHeld[DINGOO_BUTTON_UP] || Input->KeyboardHeld[SDLK_UP]))
        {
            Y-=2;
            if (Y < 10)
            {
                Y = 10;
                WorldParts->MoveAllDown();
            }

        }

        if ((Input->KeyboardHeld[DINGOO_BUTTON_DOWN] || Input->KeyboardHeld[SDLK_DOWN] ))
        {
            Y+=2;
            if(Y > 229)
            {
                Y = 229;
                WorldParts->MoveAllUp();
            }
        }

        if ( (Input->KeyboardHeld[DINGOO_BUTTON_LEFT]  || Input->KeyboardHeld[SDLK_LEFT]))
        {
            if (X > 0)
                X-=2;

        }

        if ((Input->KeyboardHeld[DINGOO_BUTTON_RIGHT]|| Input->KeyboardHeld[SDLK_RIGHT] ))
        {
            if (X < 320)
                X+=2;

        }

        SDL_FillRect(Buffer,NULL,SDL_MapRGB(Buffer->format,0,0,0));

        WorldParts->Draw(Buffer);

        PutPixel(Buffer,X+1,Y,SDL_MapRGB(Buffer->format,255,255,255));
        PutPixel(Buffer,X-1,Y,SDL_MapRGB(Buffer->format,255,255,255));
        PutPixel(Buffer,X,Y+1,SDL_MapRGB(Buffer->format,255,255,255));
        PutPixel(Buffer,X,Y-1,SDL_MapRGB(Buffer->format,255,255,255));
        PutPixel(Buffer,X+2,Y,SDL_MapRGB(Buffer->format,128,128,128));
        PutPixel(Buffer,X-2,Y,SDL_MapRGB(Buffer->format,128,128,128));
        PutPixel(Buffer,X,Y+2,SDL_MapRGB(Buffer->format,128,128,128));
        PutPixel(Buffer,X,Y-2,SDL_MapRGB(Buffer->format,128,128,128));
        PutPixel(Buffer,X+3,Y,SDL_MapRGB(Buffer->format,255,255,255));
        PutPixel(Buffer,X-3,Y,SDL_MapRGB(Buffer->format,255,255,255));
        PutPixel(Buffer,X,Y+3,SDL_MapRGB(Buffer->format,255,255,255));
        PutPixel(Buffer,X,Y-3,SDL_MapRGB(Buffer->format,255,255,255));
        PutPixel(Buffer,X,Y,SDL_MapRGB(Buffer->format,128,128,128));

        sprintf(Text,"Map Position: %d",WorldParts->MapPosition);
        WriteText(Buffer,font,Text,strlen(Text),0,0,0,Color);
        WorldParts->DrawItemInfoFromPosAt(Buffer,font,X,Y,0,20,Color);
        sprintf(Text,"%d %d",currentfps,WorldParts->ItemCount);
        WriteText(Buffer,font,Text,strlen(Text),0,200,0,Color);

        SDL_BlitSurface(Buffer,NULL,Screen,NULL);
        SDL_Flip(Screen);
        SDL_framerateDelay(&Fpsman);

        framecounter++;
        sprintf(Text,"%d %d",currentfps,WorldParts->ItemCount);

        if (Time1 + 1000 < SDL_GetTicks())
        {
            currentfps = framecounter;
            framecounter = 0;
            Time1 = SDL_GetTicks();
        }
    }
    delete Input;

}

void Game()
{

    const int numtest = 100;
    char Text[100];
    SDL_Rect Rect;
    SDL_Color Color = {255,255,255,255};
    CInput *Input = new CInput(5);
    int framecounter = 0, currentfps=0;
    int Time1 = 0;
    sprintf(Text,"0");
    WorldParts->AddPlayer(IMGTriangle,160,220);
    WorldParts->MapInc = 1;
    WorldParts->SetCanMove(true);
    WorldParts->SetLeveleditorMode(false);
    while (GameState == GSGame)
    {

        Input->Update();
        if (Input->Ready() && (Input->KeyboardHeld[DINGOO_BUTTON_SELECT] || Input->KeyboardHeld[SDLK_ESCAPE] ||Input->SpecialsHeld[SPECIAL_QUIT_EV]) )
            GameState = GSQuit;

        if ((Input->KeyboardHeld[DINGOO_BUTTON_UP] || Input->KeyboardHeld[SDLK_UP]))
        {
           WorldParts->PlayerMoveUp(2);

        }

        if ((Input->KeyboardHeld[DINGOO_BUTTON_DOWN] || Input->KeyboardHeld[SDLK_DOWN] ))
        {
            WorldParts->PlayerMoveDown(2);

        }

        if ( (Input->KeyboardHeld[DINGOO_BUTTON_LEFT]  || Input->KeyboardHeld[SDLK_LEFT]))
        {
            WorldParts->PlayerMoveLeft(2);

        }

        if ((Input->KeyboardHeld[DINGOO_BUTTON_RIGHT]|| Input->KeyboardHeld[SDLK_RIGHT] ))
        {
            WorldParts->PlayerMoveRight(2);
        }

        if (Input->Ready() && (Input->KeyboardHeld[DINGOO_BUTTON_A]|| Input->KeyboardHeld[SDLK_LCTRL] ))
        {
            WorldParts->PlayerShoot();
            Input->Delay();
        }

        WorldParts->Move();
        WorldParts->Draw(Buffer);

        sprintf(Text,"%d %d",currentfps,WorldParts->ItemCount);
        WriteText(Buffer,font,Text,strlen(Text),0,200,0,Color);

        SDL_BlitSurface(Buffer,NULL,Screen,NULL);
        SDL_Flip(Screen);
        SDL_framerateDelay(&Fpsman);

        framecounter++;

        if (Time1 + 1000 < SDL_GetTicks())
        {
            currentfps = framecounter;
            framecounter = 0;
            Time1 = SDL_GetTicks();
        }
    }
    delete Input;
    WorldParts->RemoveAll();
}

int main(int argc, char **argv)
{
    SDL_Surface *Tmp;
    int Backlight;
	//mtrace();
	if (SDL_Init(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO | SDL_INIT_AUDIO ) == 0)
	{
		printf("SDL Succesfully initialized\n");

        Screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT,16, SDL_SWSURFACE );
        if(Screen)
		{
		    printf("Succesfully Set %dx%dx16\n",WINDOW_WIDTH,WINDOW_HEIGHT);
		    Tmp = SDL_CreateRGBSurface(SDL_SWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
			Buffer = SDL_DisplayFormat(Tmp);
			SDL_FreeSurface(Tmp);
			if (Buffer)
			{
                printf("Succesfully Created Buffer\n");
                SDL_ShowCursor(SDL_DISABLE);
                    if (Mix_OpenAudio(22050,AUDIO_S16,MIX_DEFAULT_CHANNELS,1024) < 0)
                    {
                        GlobalSoundEnabled = false;
                        printf("Failed to initialise sound!\n");
                        printf("%s\n",Mix_GetError());
                    }
                    else
                    {
                        printf("Audio Succesfully initialised!\n");
                    }
                    if (TTF_Init() == 0)
                    {
                        printf("Succesfully initialized TTF\n");
                        font = TTF_OpenFont("./freefont/FreeMonoBold.ttf",10);
                        if (font)
                        {
                            printf("Succesfully Loaded fonts\n");
                            //TTF_SetFontStyle(font,TTF_STYLE_BOLD);

                            // Main game loop that loops untile the gamestate = GSQuit
                            // and calls the procedure according to the gamestate.
                            srand(unsigned(time(NULL)));
                            Backlight = getBackLight();
                            SetBacklight(80);
                            LoadSettings();
                            LoadGraphics();
                            SDL_initFramerate(&Fpsman);
                            SDL_setFramerate(&Fpsman,FPS);
                            SinCosTable = new CSinCosTable();
                            WorldParts = new CWorldParts();
                            while (GameState != GSQuit)
                            {
                                switch(GameState)
                                {
                                    case GSGame :
                                        Game();
                                        break;
                                    case GSLevelEditor:
                                        LevelEditor();
                                        break;
                                    case GSTitleScreen:
                                        GameState=GSQuit;
                                        break;
                                    case GSIntro :
                                        GameState=GSQuit;
                                        break;
                                }
                            }
                            SetBacklight(Backlight);
                            delete WorldParts;
                            delete SinCosTable;
                            UnLoadGraphics();
                            SaveSettings();
                            Mix_CloseAudio();
                            TTF_CloseFont(font);
                            font=NULL;
                        }
                        else
                        {
                            printf("Failed to Load fonts\n");
                        }
                        TTF_Quit();
                    }
                    else
                    {
                        printf("Failed to initialize TTF\n");
                    }
                SDL_FreeSurface(Buffer);
                Buffer=NULL;
			}
			else
                printf("Failed To Create Buffer\n");
            SDL_FreeSurface(Screen);
            Screen=NULL;
		}
		else
		{
			printf("Failed to Set Videomode %dx%dx16\n",WINDOW_WIDTH, WINDOW_HEIGHT);
		}
		SDL_Quit();
	}
	else
	{
		printf("Couldn't initialise SDL!\n");
	}
	// Shouldn't be needed but i just placed it again to be sure all the boardparts are free
	// Call the menu
	//chdir("/usr/gp2x");
	//execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);

	return 0;

}
